/*****************************************************************************\
 *
 * main.cpp
 *
 * main - hlavný súbor obsahujúci shadere, scénky a všetko potrebné k nim
 *
 * Authors: Pavol Rehorčík (xrehor00 _AT stud.fit.vutbr.cz)
 *
 * Contributors: Paul Bourke
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <assert.h>
#include <math.h>
#include <GL/glew.h>
#include <GL/wglew.h>
#include <GL/glut.h>
#include "ApacheAh64.h"
#include "F14.h"
#include "type9shuttlev2.h"

#define POCET_TEXTUR            8 // pouzivam tolkoto textur
#define TEXTURA_ZEME            0 // index v poli (0.) textury kocky
#define TEXTURA_STIEN           1 // index v poli (1.) textury stvorcov na miestnost, tehlova textura
#define TEXTURA_PODLAHY         2 // ...
#define TEXTURA_STIEN_NORMALOVA 3
#define TEXTURA_STIEN_VYSKOVA   4
#define TEXTURA_TEHLY           5
#define TEXTURA_TEHLY_NORMALOVA 6
#define TEXTURA_TEHLY_VYSKOVA   7

#define PI 3.141592654
#define TWOPI 6.283185308

// -- globalne premenne
int scenka = 1; // prepinac scenky, default: 1

char FPS [15]; // pole pre ulozenie ulozenie textu pre vypis FPS

// premenne pre cas
double cas = -1;
double rozdiel = 0;
double lastUpdate = -1.0;

// rotacne premenne pozicie svetla
float x0 = 0.0, yy0 = 0.0, z0 = 1.5707963267949;
                            // pi/2
float x1 = 1.5707963267949, yy1 = 0.0, z1 = -3.1415926535898; // -4.0, 1.0, 4.0, 1.0
float x2 = 0.0, yy2 = 0.0, z2 = 1.5707963267949;
float x3 = 1.5707963267949, yy3 = 0.0, z3 = 3.1415926535898; // -4.0, 8.0, -4.0, 1.0
float x4 = 0.0, yy4 = 0.0, z4 = 1.5707963267949;
float x5 = 0.0, yy5 = 0.0, z5 = 1.5707963267949;
float x6 = 0.0, yy6 = 0.0, z6 = 1.5707963267949;

// cosiny rotacnych premennych svetla
float cosx0, cosy0, cosz0;
float cosx1, cosy1, cosz1;
float cosx2, cosy2, cosz2;
float cosx3, cosy3, cosz3;
float cosx4, cosy4, cosz4;
float cosx5, cosy5, cosz5;
float cosx6, cosy6, cosz6;
float cosx7, cosy7, cosz7;

// pre shadre
int vertexSh, fragmentSh, program;
int vertexShOfTeapotTexture, fragmentShOfTeapotTexture, programOfTeapotTexture;
int vertexShOfStien, fragmentShOfStien, programOfStien;
int vertexShOfPodlahy, fragmentShOfPodlahy, programOfPodlahy;
int vertexShOfSimplePM, fragmentShOfSimplePM, programOfSimplePM;
int vertexShOfSteepPM, fragmentShOfSteepPM, programOfSteepPM;
int vertexShOfPOM, fragmentShOfPOM, programOfPOM;
int vertexShOfPOMSS, fragmentShOfPOMSS, programOfPOMSS;

GLuint listGule = glGenLists(1); // display list

bool toggle_help = true; // toggle help
bool toggle_info = true; // toggle renderer information

bool nenacitana_texutra = false; // zobrazi hlasku, ze sa nepodarilo nacitat jednu z textur

// light 0 - white
float Light0Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light0Diffuse[]  = {1.0, 1.0, 1.0, 1.0};
float Light0Position[] = {0.0, 5.0, 0.0, 1.0};
float Light0Specular[] = {1.0, 1.0, 1.0, 1.0};

//light 1 - red
float Light1Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light1Diffuse[]  = {1.0, 0.0, 0.0, 1.0};
float Light1Position[] = {-4.0, 1.0, 4.0, 1.0};
float Light1Specular[] = {1.0, 0.0, 0.0, 1.0};

//light 2 - green
float Light2Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light2Diffuse[]  = {0.0, 1.0, 0.0, 1.0};
float Light2Position[] = {-4.0, 9.0, 4.0, 1.0};
float Light2Specular[] = {0.0, 1.0, 0.0, 1.0};

//light 3 - blue
float Light3Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light3Diffuse[]  = {0.0, 0.0, 1.0, 1.0};
float Light3Position[] = {-4.0, 8.0, -4.0, 1.0};
float Light3Specular[] = {0.0, 0.0, 1.0, 1.0};

//light 4 - violet (red + blue)
float Light4Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light4Diffuse[]  = {2.0, 0.0, 2.0, 1.0};
float Light4Position[] = {-4.0, 2, -4.0, 1.0};
float Light4Specular[] = {1.0, 0.0, 1.0, 1.0};;

//light 5 - (green + blue)
float Light5Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light5Diffuse[]  = {0.0, 1.0, 1.0, 1.0};
float Light5Position[] = {4.0, 5.0, 4.0, 1.0};
float Light5Specular[] = {0.0, 1.0, 1.0, 1.0};

//light 6 - yellow (red + green)
float Light6Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light6Diffuse[]  = {2.0, 2.0, 0.0, 1.0};
float Light6Position[] = {4.0, 6.0, -4.0, 1.0};
float Light6Specular[] = {1.0, 1.0, 0.0, 1.0};

//light 7 - again white
float Light7Ambient[]  = {0.0, 0.0, 0.0, 1.0};
float Light7Diffuse[]  = {1.0, 1.0, 1.0, 1.0};
float Light7Position[] = {0.0, 5.0, -11.0, 1.0};
float Light7Specular[] = {1.0, 1.0, 1.0, 1.0};

// materialne premenne
float MaterialAmbient[]  = {0.0, 0.0, 0.0, 1.0};
float MaterialDiffuse[]  = {0.0, 0.0, 0.0, 1.0};
float MaterialSpecular[] = {0.3, 0.3, 0.3, 1.0};
float mShininess[] = {127};

// struktura pre texturu
typedef struct _RGBIMG {
	GLuint   w;    // sirka
	GLuint   h;    // vyska
	GLubyte* data; // data, pixely
} RGBIMG;

// struktura pre polohu gule
typedef struct {
  float x;
  float y;
  float z;
}XYZ;

GLuint textura_id[POCET_TEXTUR]; // pole textur

// -- implementacia shaderov
const GLchar* shaderSrcVertex[] = {
  "varying vec3 vertex_normal;"
  "varying vec3 vertex_position;"
  "void main()"
  "{"
  "  vertex_normal = gl_NormalMatrix * gl_Normal;"
  "  vertex_position =  gl_Vertex.xyz;"
  "  gl_FrontColor = gl_Color;" // farba glColor

  "  gl_Position = ftransform();"
  "}"
};

const GLchar* shaderSrcFragment[] = {
  "varying vec3 vertex_normal;"
  "varying vec3 vertex_position;"

  "vec3 vertex_normal_normalized = normalize(vertex_normal);"
  "vec3 cameraVector = normalize(vec3(0.0, 2.0, 15.0) - vertex_position);"

  "vec4 funkcia(vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse, vec4 lightSpecular)"
  "{"
  "  vec3 lightVector = lightPosition - vertex_position;"
  "  float distance = length(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(vertex_normal_normalized, normalize(lightVector)), 0.0);"
  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation;"

  "  float specularPower = 0.0;"
  "  if(skalarnySucin > 0.0)"
  "  {"
  "    vec3 halfVector = normalize(normalize(lightVector) + cameraVector);"
  "    float skalarnyHalf = max(dot(vertex_normal_normalized, halfVector), 0.0);"
  "    specularPower = 0.5 * pow(skalarnyHalf, gl_FrontMaterial.shininess) * attenuation;"
  "  }"
  "  vec4 specular_color = gl_FrontMaterial.specular * lightSpecular * specularPower;"

  "  return gl_Color * diffuse_color + specular_color;"
  "}"

  "void main()"
  "{"
  "  if (gl_FrontFacing == false) {vertex_normal_normalized = -vertex_normal_normalized;}"
  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse, gl_LightSource[i].specular);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"
  "  gl_FragColor = vysledna_farba;"
  "}"
};

const GLchar* shaderSrcVertexOfTeapotTexture[] = {
  "varying vec3 vertex_normal;" // normala vertexu
  "varying vec3 vertex_position;"
  "void main()"
  "{"
  "  vertex_normal = gl_NormalMatrix * gl_Normal;"
  //"  vertex_position =  vec3(gl_ModelViewMatrix * gl_Vertex);"
  "  vertex_position = gl_Vertex.xyz;"
  "  gl_TexCoord[0] = gl_MultiTexCoord0;" // texturovanie

  "  gl_Position = ftransform();"
  "}"
};

const GLchar* shaderSrcFragmentOfTeapotTexture[] = {
  "varying vec3 vertex_normal;" // normala vertexu
  "varying vec3 vertex_position;"
  "uniform sampler2D tex;" // uniform pre texturu
  "vec4 tex_color = texture2D(tex, gl_TexCoord[0].st);" // farba otexturovaneho pixelu
  "vec3 vertex_normal_normalized = normalize(vertex_normal);"
  "vec3 cameraVector = normalize(vec3(0.0, 2.0, 15.0) - vertex_position);"

  "vec4 funkcia(vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse, vec4 lightSpecular)"
  "{"
  "  vec3 lightVector = lightPosition - vertex_position;"
  "  float distance = length(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(vertex_normal_normalized, normalize(lightVector)), 0.0);"
  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation;"

  "  float specularPower = 0.0;"
  "  if(skalarnySucin > 0.0)"
  "  {"
  "    vec3 halfVector = normalize(normalize(lightVector) + cameraVector);"
  "    float skalarnyHalf = max(dot(vertex_normal_normalized, halfVector), 0.0);"
  "    specularPower = 0.5 * pow(skalarnyHalf, gl_FrontMaterial.shininess) * attenuation;"
  "  }"
  "  vec4 specular_color = gl_FrontMaterial.specular * lightSpecular * specularPower;"

  "  return tex_color * diffuse_color + specular_color;"
  "}"

  "void main()"
  "{"
  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse, gl_LightSource[i].specular);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"
  "  gl_FragColor = vysledna_farba;"
  "}"
};

const GLchar* shaderSrcVertexOfStien[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "attribute vec3 tangent;"

  "void main()"
  "{"
  "  vec3 vertex_normal = normalize(gl_NormalMatrix * gl_Normal);" // normalizovana normala vertexu v mojich suradniciach
  "  vec3 tangent_normalized = normalize(gl_NormalMatrix * tangent);"
  "  vec3 binormal = cross(vertex_normal, tangent_normalized);"

  "  vec3 vertex_position =  gl_Vertex.xyz;" // pozicia vertexu, sa uklada do premennej, nech sa nevola 8x

  // --- TBN Matrix
  "  mat3 TBNMatrix = mat3(tangent_normalized, binormal, vertex_normal);"

  // --- Light Vectors
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    lightVector[i] = (gl_LightSource[i].position.xyz - vertex_position) * TBNMatrix;"
  "  }"

  // --- Eye Vector
  "  eyeVector = (vec3(0.0, 2.0, 15.0) - vertex_position) * TBNMatrix;"

  "  gl_TexCoord[1] = gl_MultiTexCoord1;" // texturovanie

  "  gl_Position = ftransform();"
  "}"
};

const GLchar* shaderSrcFragmentOfStien[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "uniform sampler2D myTexture2;"
  "uniform sampler2D normalMap;"
  "uniform sampler2D heightMap;"
  "vec2 newCoords;"
  "vec3 normaly_z_mapy;"
  "vec4 tex_color;"

  "vec4 funkcia(vec3 lightVector, vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse)"
  "{"
  "  float distance = length(lightVector);"
  "  lightVector = normalize(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(normaly_z_mapy, lightVector), 0.0);"

  "  float selfShadow = 0.0;"
  "  if(skalarnySucin > 0)"
  "  {"
  "    const int numShadowSteps = 30;"
  "    float step = 1.0 / numShadowSteps;"               // bump scale
  "    vec2 LDelta = vec2(lightVector.x, -lightVector.y) * 0.03 / (lightVector.z * numShadowSteps);"
  "    vec2 offset = gl_TexCoord[1].st;"
  "    vec4 NB = texture2D(heightMap, offset);"
  "    float height = NB.r + step;" // preskocime 1. interaciu aby sme sa vyhli niektorym chybam v tieni
  "    while(((NB.r < height) && (height < 1)))"
  "    {"
  "      height = height + step; "
  "      offset = offset + LDelta;"
  "      NB = texture2D(heightMap, offset);"
  "    }"
  "    if (NB.r < height)"
  "      {selfShadow = 1.0;}"
  "  }"

  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation * selfShadow;"
    /** Na steny zrkadlova zloska nepatri!*/
  "  return tex_color * diffuse_color;"
  "}"

  "void main()"
  "{"
  "  const int numSteps = 30;"
  "  float height = 1.0;"
  "  float step = 1.0 / numSteps;"
  "  vec2 offset = gl_TexCoord[1].st;"
  "  float h = texture2D(heightMap, offset).r;"       // bump scale
  "  vec2 delta = vec2(-eyeVector.x, eyeVector.y) * 0.03 / (eyeVector.z * numSteps);"
  "  while(h < height)"
  "  {"
  "    height = height - step;"
  "    offset = offset + delta;"
  "    h = texture2D(heightMap, offset).r;"
  "  }"

  "  vec2 prev = offset - delta;"
  "  float hPrev = texture2D(heightMap, prev).r - (height + step);"
  "  float hCur = h - height;"
  "  float weight = hCur / (hCur - hPrev);"
  "  offset = weight * prev + (1.0 - weight) * offset;"

  "  normaly_z_mapy = texture2D(normalMap, offset).rgb * 2.0 - 1.0;" // nacitanie normal z normalovej mapy, bump mapping

  "  tex_color = texture2D(myTexture2, offset);" // texturovanie s novymi tex coords

  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(lightVector[i], gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"

  "  gl_FragColor = vysledna_farba;"
  "}"
};

const GLchar* shaderSrcVertexOfPodlahy[] = {
  "varying vec3 vertex_normal;" // normala vertexu
  "varying vec3 vertex_position;"
  "void main()"
  "{"
  "  vertex_normal = gl_NormalMatrix * gl_Normal;" // normala vertexu v mojich suradniciach
  "  vertex_position =  gl_Vertex.xyz;" // pozicia vertexu
  "  gl_TexCoord[2] = gl_MultiTexCoord2;" // texturovanie

  "  gl_Position = ftransform();"
  "}"
};

const GLchar* shaderSrcFragmentOfPodlahy[] = {
  "varying vec3 vertex_normal;" // normala vertexu
  "varying vec3 vertex_position;" // pozicia vertexu
  "uniform sampler2D myTexture3;"
  "vec4 tex_color = texture2D(myTexture3, vec2(gl_TexCoord[2]));" // texturovanie
  "vec3 vertex_normal_normalized = normalize(vertex_normal);"
  "vec3 cameraVector = normalize(vec3(0.0, 2.0, 15.0) - vertex_position);"

  "vec4 funkcia(vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse, vec4 lightSpecular)"
  "{"
  "  vec3 lightVector = lightPosition - vertex_position;"
  "  float distance = length(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(vertex_normal_normalized, normalize(lightVector)), 0.0);"
  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation;"

  "  float specularPower = 0.0;"
  "  if(skalarnySucin > 0.0)"
  "  {"
  "    vec3 halfVector = normalize(normalize(lightVector) + cameraVector);"
  "    float skalarnyHalf = max(dot(vertex_normal_normalized, halfVector), 0.0);"
  "    specularPower = pow(skalarnyHalf, gl_FrontMaterial.shininess) * attenuation;"
  "  }"
  "  vec4 specular_color = gl_FrontMaterial.specular * lightSpecular * specularPower;"

  "  return tex_color * diffuse_color + specular_color;"
  "}"

  "void main()"
  "{"
  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse, gl_LightSource[i].specular);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"
  "  gl_FragColor = vysledna_farba;"
  "}"
};

const GLchar* shaderSrcVertexOfSimplePM[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "attribute vec3 tangent;"

  "void main()"
  "{"
  "  vec3 vertex_normal = normalize(gl_NormalMatrix * gl_Normal);" // normalizovana normala vertexu v mojich suradniciach
  "  vec3 tangent_normalized = normalize(gl_NormalMatrix * tangent);"
  "  vec3 binormal = cross(vertex_normal, tangent_normalized);"

  "  vec3 vertex_position =  gl_Vertex.xyz;" // pozicia vertexu, sa uklada do premennej, nech sa nevola 8x

  // --- TBN Matrix
  "  mat3 TBNMatrix = mat3(tangent_normalized, binormal, vertex_normal);"

  // --- Light Vectors
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    lightVector[i] = (gl_LightSource[i].position.xyz - vertex_position) * TBNMatrix;"
  "  }"

  // --- Eye Vector
  "  eyeVector = (vec3(0.0, 2.0, 15.0) - vertex_position) * TBNMatrix;"

  "  gl_TexCoord[5] = gl_MultiTexCoord5;" // texturovanie

  "  gl_Position = ftransform();"
  "}"
};

const GLchar* shaderSrcFragmentOfSimplePM[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "uniform sampler2D stropColor;"
  "uniform sampler2D normalMap;"
  "uniform sampler2D heightMap;"
  "vec2 newCoords;"
  "vec3 normaly_z_mapy;"
  "vec4 tex_color;"

  "vec4 funkcia(vec3 lightVector, vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse)"
  "{"
  "  float distance = length(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(normaly_z_mapy, normalize(lightVector)), 0.0);"
  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation;"
    /** Na steny zrkadlova zloska nepatri!*/
  "  return tex_color * diffuse_color;"
  "}"

  "void main()"
  "{"
  "  float height = texture2D(heightMap, gl_TexCoord[5].st).r;"
  "  vec2 scaleBias = vec2(0.04, 0.02);"
  "  float v = height * scaleBias.r - scaleBias.g;"
  "  newCoords = gl_TexCoord[5].st + (v * (normalize(eyeVector).xy));"

  "  normaly_z_mapy = texture2D(normalMap, newCoords).rgb * 2.0 - 1.0;" // nacitanie normal z normalovej mapy, bump mapping

  "  tex_color = texture2D(stropColor, newCoords);" // texturovanie s novymi tex coords

  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(lightVector[i], gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"

  "  gl_FragColor = vysledna_farba;"
  "}"
};

const GLchar* shaderSrcVertexOfSteepPM[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "attribute vec3 tangent;"

  "void main()"
  "{"
  "  vec3 vertex_normal = normalize(gl_NormalMatrix * gl_Normal);" // normalizovana normala vertexu v mojich suradniciach
  "  vec3 tangent_normalized = normalize(gl_NormalMatrix * tangent);"
  "  vec3 binormal = cross(vertex_normal, tangent_normalized);"

  "  vec3 vertex_position =  gl_Vertex.xyz;" // pozicia vertexu, sa uklada do premennej, nech sa nevola 8x

  // --- TBN Matrix
  "  mat3 TBNMatrix = mat3(tangent_normalized, binormal, vertex_normal);"

  // --- Light Vectors
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    lightVector[i] = (gl_LightSource[i].position.xyz - vertex_position) * TBNMatrix;"
  "  }"

  // --- Eye Vector
  "  eyeVector = (vec3(0.0, 2.0, 15.0) - vertex_position) * TBNMatrix;"

  "  gl_TexCoord[5] = gl_MultiTexCoord5;" // texturovanie

  "  gl_Position = ftransform();"
  "}"
};

const GLchar* shaderSrcFragmentOfSeepPM[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "uniform sampler2D stropColor;"
  "uniform sampler2D normalMap;"
  "uniform sampler2D heightMap;"
  "vec2 newCoords;"
  "vec3 normaly_z_mapy;"
  "vec4 tex_color;"

  "vec4 funkcia(vec3 lightVector, vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse)"
  "{"
  "  float distance = length(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(normaly_z_mapy, normalize(lightVector)), 0.0);"

  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation;"
    /** Na steny zrkadlova zloska nepatri!*/
  "  return tex_color * diffuse_color;"
  "}"

  "void main()"
  "{"
  "  const int numSteps = 30;" // pocet vrstiev
  "  float height = 1.0;" // pociatocna najvrchnejsia vrstva
  "  float step = 1.0 / numSteps;" // vzdialenost medzi vrstvami
  "  vec2 offset = gl_TexCoord[5].st;" // pociatocny offset
  "  float h = texture2D(heightMap, offset).r;" // vyska z mapy v pociatocnom offsete
  "  vec2 delta = vec2(-eyeVector.x, eyeVector.y) * 0.03 / (eyeVector.z * numSteps);" // posuv medzi dvoma vrstvami
  "  while(h < height)"                            // 0.03 je bump scale
  "  {" // prechadzanie po vrstvach, hladanie najvhodnejsieho offsetu
  "    height = height - step;"
  "    offset = offset + delta;"
  "    h = texture2D(heightMap, offset).r;" // vyska v tomto offsete
  "  }"

  "  normaly_z_mapy = texture2D(normalMap, offset).rgb * 2.0 - 1.0;" // nacitanie normal z normalovej mapy, bump mapping

  "  tex_color = texture2D(stropColor, offset);" // texturovanie s novymi tex coords

  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(lightVector[i], gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"

  "  gl_FragColor = vysledna_farba;"
  "}"
};

const GLchar* shaderSrcVertexOfPOM[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "attribute vec3 tangent;"

  "void main()"
  "{"
  "  vec3 vertex_normal = normalize(gl_NormalMatrix * gl_Normal);" // normalizovana normala vertexu v mojich suradniciach
  "  vec3 tangent_normalized = normalize(gl_NormalMatrix * tangent);"
  "  vec3 binormal = cross(vertex_normal, tangent_normalized);"

  "  vec3 vertex_position =  gl_Vertex.xyz;" // pozicia vertexu, sa uklada do premennej, nech sa nevola 8x

  // --- TBN Matrix
  "  mat3 TBNMatrix = mat3(tangent_normalized, binormal, vertex_normal);"

  // --- Light Vectors
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    lightVector[i] = (gl_LightSource[i].position.xyz - vertex_position) * TBNMatrix;"
  "  }"

  // --- Eye Vector
  "  eyeVector = (vec3(0.0, 2.0, 15.0) - vertex_position) * TBNMatrix;"

  "  gl_TexCoord[5] = gl_MultiTexCoord5;" // texturovanie

  "  gl_Position = ftransform();"
  "}"
};

const GLchar* shaderSrcFragmentOfPOM[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "uniform sampler2D stropColor;"
  "uniform sampler2D normalMap;"
  "uniform sampler2D heightMap;"
  "vec2 newCoords;"
  "vec3 normaly_z_mapy;"
  "vec4 tex_color;"

  "vec4 funkcia(vec3 lightVector, vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse)"
  "{"
  "  float distance = length(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(normaly_z_mapy, normalize(lightVector)), 0.0);"

  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation;"
    /** Na steny zrkadlova zloska nepatri!*/
  "  return tex_color * diffuse_color;"
  "}"

  "void main()"
  "{"
  "  const int numSteps = 30;"
  "  float height = 1.0;"
  "  float step = 1.0 / numSteps;"
  "  vec2 offset = gl_TexCoord[5].st;"
  "  float h = texture2D(heightMap, offset).r;"       // bump scale
  "  vec2 delta = vec2(-eyeVector.x, eyeVector.y) * 0.03 / (eyeVector.z * numSteps);"
  "  while(h < height)"
  "  {"
  "    height = height - step;"
  "    offset = offset + delta;"
  "    h = texture2D(heightMap, offset).r;"
  "  }"

  "  vec2 prev = offset - delta;"
  "  float hPrev = texture2D(heightMap, prev).r - (height + step);"
  "  float hCur = h - height;"
  "  float weight = hCur / (hCur - hPrev);"
  "  offset = weight * prev + (1.0 - weight) * offset;"

  "  normaly_z_mapy = texture2D(normalMap, offset).rgb * 2.0 - 1.0;" // nacitanie normal z normalovej mapy, bump mapping

  "  tex_color = texture2D(stropColor, offset);" // texturovanie s novymi tex coords

  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(lightVector[i], gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"

  "  gl_FragColor = vysledna_farba;"
  "}"
};

const GLchar* shaderSrcVertexOfPOMSS[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "attribute vec3 tangent;"

  "void main()"
  "{"
  "  vec3 vertex_normal = normalize(gl_NormalMatrix * gl_Normal);" // normalizovana normala vertexu v mojich suradniciach
  "  vec3 tangent_normalized = normalize(gl_NormalMatrix * tangent);"
  "  vec3 binormal = cross(vertex_normal, tangent_normalized);"

  "  vec3 vertex_position =  gl_Vertex.xyz;" // pozicia vertexu, sa uklada do premennej, nech sa nevola 8x

  // --- TBN Matrix
  "  mat3 TBNMatrix = mat3(tangent_normalized, binormal, vertex_normal);"

  // --- Light Vectors
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    lightVector[i] = (gl_LightSource[i].position.xyz - vertex_position) * TBNMatrix;"
  "  }"

  // --- Eye Vector
  "  eyeVector = (vec3(0.0, 2.0, 15.0) - vertex_position) * TBNMatrix;"

  "  gl_TexCoord[5] = gl_MultiTexCoord5;" // texturovanie

  "  gl_Position = ftransform();"
  "}"
};
/*
const GLchar* shaderSrcFragmentOfPOMSS[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "uniform sampler2D stropColor;"
  "uniform sampler2D normalMap;"
  "uniform sampler2D heightMap;"
  "vec3 normaly_z_mapy;"
  "vec4 tex_color;"

  "vec4 funkcia(vec3 lightVector, vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse)"
  "{"
  "  float distance = length(lightVector);"
  "  lightVector = normalize(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(normaly_z_mapy, lightVector), 0.0);"

  "  float selfShadow = 0.0;"
  "  if(skalarnySucin > 0)"
  "  {"
  "    const int numShadowSteps = 100;"
  "    float step = 1.0 / numShadowSteps;"           // bump scale
  "    vec2 LDelta = vec2(lightVector.x, -lightVector.y) * 0.1 / (lightVector.z * numShadowSteps);"
  "    vec2 offset = gl_TexCoord[5].st;"
  "    vec4 NB = texture2D(heightMap, offset);"
  "    float height = NB.r + step * 0.1;" // preskocime 1. interaciu aby sme sa vyhli niektorym chybam v tieni
  "    while(((NB.r < height) && (height < 1)))"
  "    {"
  "      height = height + step; "
  "      offset = offset + LDelta;"
  "      NB = texture2D(heightMap, offset);"
  "    }"

  "    if (NB.r < height)"
  "      {selfShadow = 1.0;}"
  "  }"

  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation * selfShadow;"

  "  return tex_color * diffuse_color;"
  "}"

  "void main()"
  "{"
  "  const int numSteps = 100;"
  "  float height = 1.0;"
  "  float step = 1.0 / numSteps;"
  "  vec2 offset = gl_TexCoord[5].st;"
  "  float h = texture2D(heightMap, offset).r;"       // bump scale
  "  vec2 delta = vec2(-eyeVector.x, eyeVector.y) * 0.1 / (eyeVector.z * numSteps);"
  "  while(h < height)"
  "  {"
  "    height = height - step;"
  "    offset = offset + delta;"
  "    h = texture2D(heightMap, offset).r;"
  "  }"

  "  vec2 prev = offset - delta;"
  "  float hPrev = texture2D(heightMap, prev).r - (height + step);"
  "  float hCur = h - height;"
  "  float weight = hCur / (hCur - hPrev);"
  "  offset = weight * prev + (1.0 - weight) * offset;"

  "  normaly_z_mapy = texture2D(normalMap, offset).rgb * 2.0 - 1.0;" // nacitanie normal z normalovej mapy, bump mapping

  "  tex_color = texture2D(stropColor, offset);" // texturovanie s novymi tex coords

  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 1; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(lightVector[i], gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"

  "  gl_FragColor = vysledna_farba;"
  "}"
};*/

const GLchar* shaderSrcFragmentOfPOMSS[] = {
  "varying vec3 lightVector[8];"
  "varying vec3 eyeVector;"
  "uniform sampler2D stropColor;"
  "uniform sampler2D normalMap;"
  "uniform sampler2D heightMap;"
  "vec3 normaly_z_mapy;"
  "vec4 tex_color;"

  "vec4 funkcia(vec3 lightVector, vec3 lightPosition, float constantAtt, float linearAtt, float quadraticAtt, vec4 lightDiffuse)"
  "{"
  "  float distance = length(lightVector);"
  "  lightVector = normalize(lightVector);"
  "  float attenuation = 1.0 / (constantAtt + linearAtt * distance + quadraticAtt * distance * distance);"
  "  float skalarnySucin = max(dot(normaly_z_mapy, lightVector), 0.0);"

  "  vec2 vLightRayTS = vec2(lightVector.x, -lightVector.y) * 0.03;" //scale
  "  float ShadowSoftening = 1.0;"

  "  float fOcclusionShadow;"
  "  const int numShadowSteps = 30;"
  "  float height = 1.0;"
  "  float step = 1.0 / numShadowSteps;"
  "  vec2 offset = gl_TexCoord[5].st;"
  "  float h = texture2D(heightMap, offset).r;"       // bump scale
  "  vec2 delta = vec2(-eyeVector.x, eyeVector.y) * 0.03 / (eyeVector.z * numShadowSteps);"
  "  while(h < height)"
  "  {"
  "    height = height - step;"
  "    offset = offset + delta;"
  "    h = texture2D(heightMap, offset).r;"
  "  }"

  "  float sh0 =  texture2D( heightMap, offset.st).r;"
  "  float shA = (texture2D( heightMap, offset.st + vLightRayTS * 0.88 * 4.0).r - sh0 - 0.88 ) * 4.22 * ShadowSoftening;"
  "  float sh9 = (texture2D( heightMap, offset.st + vLightRayTS * 0.77 * 4.0).r - sh0 - 0.77 ) * 4.33 * ShadowSoftening;"
  "  float sh8 = (texture2D( heightMap, offset.st + vLightRayTS * 0.66 * 4.0).r - sh0 - 0.66 ) * 4.44 * ShadowSoftening;"
  "  float sh7 = (texture2D( heightMap, offset.st + vLightRayTS * 0.55 * 4.0).r - sh0 - 0.55 ) * 4.55 * ShadowSoftening;"
  "  float sh6 = (texture2D( heightMap, offset.st + vLightRayTS * 0.44 * 4.0).r - sh0 - 0.44 ) * 4.66 * ShadowSoftening;"
  "  float sh5 = (texture2D( heightMap, offset.st + vLightRayTS * 0.33 * 4.0).r - sh0 - 0.33 ) * 4.77 * ShadowSoftening;"
  "  float sh4 = (texture2D( heightMap, offset.st + vLightRayTS * 0.22 * 4.0).r - sh0 - 0.22 ) * 4.88 * ShadowSoftening;"
  "  fOcclusionShadow = 1.0 - max( max( max( max( max( max( shA, sh9 ), sh8 ), sh7 ), sh6 ), sh5 ), sh4 );"
  "  fOcclusionShadow = fOcclusionShadow * 0.45;"

  "  vec4 diffuse_color = gl_FrontMaterial.diffuse * lightDiffuse * skalarnySucin * attenuation * fOcclusionShadow;"

  "  return tex_color * diffuse_color;"
  "}"

  "void main()"
  "{"
  "  const int numSteps = 30;"
  "  float height = 1.0;"
  "  float step = 1.0 / numSteps;"
  "  vec2 offset = gl_TexCoord[5].st;"
  "  float h = texture2D(heightMap, offset).r;"       // bump scale
  "  vec2 delta = vec2(-eyeVector.x, eyeVector.y) * 0.03 / (eyeVector.z * numSteps);"
  "  while(h < height)"
  "  {"
  "    height = height - step;"
  "    offset = offset + delta;"
  "    h = texture2D(heightMap, offset).r;"
  "  }"

  "  vec2 prev = offset - delta;" // predosly bod, navrat o jedna
  "  float hPrev = texture2D(heightMap, prev).r - (height + step);" // predosle h
  "  float hCur = h - height;" // aktualne h
  "  float weight = hCur / (hCur - hPrev);" // váha
  "  offset = weight * prev + (1.0 - weight) * offset;" // interpolácia

  "  normaly_z_mapy = texture2D(normalMap, offset).rgb * 2.0 - 1.0;" // nacitanie normal z normalovej mapy, bump mapping

  "  tex_color = texture2D(stropColor, offset);" // texturovanie s novymi tex coords

  "  vec4 ciastkova_farba[8];"
  "  vec4 vysledna_farba = vec4(0.0);"
  "  for(int i = 0; i < 8; i++)"
  "  {"
  "    ciastkova_farba[i] = funkcia(lightVector[i], gl_LightSource[i].position.xyz, gl_LightSource[i].constantAttenuation, gl_LightSource[i].linearAttenuation, gl_LightSource[i].quadraticAttenuation, gl_LightSource[i].diffuse);"
  "    vysledna_farba = vysledna_farba + ciastkova_farba[i];"
  "  }"

  "  gl_FragColor = vysledna_farba;"
  "}"
};

// -- pomocne funckie

// funkcia na ziskanie casu pod Windows
double getTime()
{
  #define DOUBLE0x100000000 4294967296.

  static bool initialized = false;
  static double frequency;
  static double startupTime;

  if (!initialized)
  {
    initialized = true;
    LARGE_INTEGER v;
    bool available = (QueryPerformanceFrequency(&v) != 0);
    if (!available)
    {
      assert(0 && "QueryPerformanceFrequency() does not work. This should not ever happen since QueryPerformanceFrequency works on all known systems as far as developers of this code know.");
      exit(EXIT_FAILURE);
    }

    frequency = 1./(DOUBLE0x100000000*v.HighPart + v.LowPart);
    bool b = (QueryPerformanceCounter(&v) != 0);
    assert(b && "QueryPerformanceCounter() failed although QueryPerformanceFrequency() worked.");
    startupTime = (DOUBLE0x100000000*v.HighPart + v.LowPart)*frequency;

    return 0.;
  }

  LARGE_INTEGER v;

  bool b = (QueryPerformanceCounter(&v) != 0);
  assert(b && "QueryPerformanceCounter() failed although QueryPerformanceFrequency() worked.");
  return (DOUBLE0x100000000*v.HighPart + v.LowPart)*frequency - startupTime;
}

// nacitanie RAW
bool loadRgbImage(const char* file_name, int w, int h, RGBIMG* refimg)
{
	GLuint   sz;    // Odlzka v bytoch datoveho pola
	FILE*    file;  // subor na disku
	long     fsize; // velkost suoru v bytoch
	GLubyte* p;     // pomocny pointer

	refimg->w = (GLuint) w;
	refimg->h = (GLuint) h;
	sz = (((3*refimg->w+3)>>2)<<2)*refimg->h;
	refimg->data = new GLubyte [sz];
	if (refimg->data == NULL)
	  return false;

	/*otvori subor a nacita pixely*/
	file = fopen(file_name , "rb");
	if (!file)
	  return false;
	fseek(file, 0L, SEEK_END);
	fsize = ftell(file);
	if (fsize != (long)sz)
	{
		fclose(file);
		return false;
	}
	fseek(file, 0L, SEEK_SET);
	p = refimg->data;
	while (fsize > 0)
	{
		fread(p, 1, 1, file);
		p++;
		fsize--;
	}
	fclose(file);
	return true;
}

/*** kod pana Paula Bourka, ak niekomu vadi tento kus kodu, moze sa vklude odstranit (tento kus kodu)*/
// vytvorenie gule so stredom v c, polomerom r a presnostou n
void CreateSphere(XYZ c, double r, int n)
{
   int i,j;
   double theta1,theta2,theta3;
   XYZ e,p;

   if (r < 0)
      r = -r;
   if (n < 0)
      n = -n;
   if (n < 4 || r <= 0)
   {
      glBegin(GL_POINTS);
        glVertex3f(c.x,c.y,c.z);
      glEnd();
      return;
   }

   for (j=0;j<n/2;j++)
   {
      theta1 = j * TWOPI / n - PI/2;
      theta2 = (j + 1) * TWOPI / n - PI/2;

    glNewList(listGule, GL_COMPILE);
      glBegin(GL_TRIANGLE_STRIP);
      for (i=0;i<=n;i++)
      {
         theta3 = i * TWOPI / n;

         e.x = cos(theta2) * cos(theta3);
         e.y = sin(theta2);
         e.z = cos(theta2) * sin(theta3);
         p.x = c.x + r * e.x;
         p.y = c.y + r * e.y;
         p.z = c.z + r * e.z;

         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
         glVertex3f(p.x,p.y,p.z);

         e.x = cos(theta1) * cos(theta3);
         e.y = sin(theta1);
         e.z = cos(theta1) * sin(theta3);
         p.x = c.x + r * e.x;
         p.y = c.y + r * e.y;
         p.z = c.z + r * e.z;

         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*j/(double)n);
         glVertex3f(p.x,p.y,p.z);
      }
      glEnd();
    glEndList();
   }
}
/*** koniec kodu pana Paula Bourka*/

// vypisuje FPS prekonvertovane na text
void renderBitmapString(float x, float y, float z, const char *string)
{
  //glDisable(GL_LIGHTING);
  glColor3f(0.0, 1.0, 1.0);
  const char *c;
  glRasterPos2f(x, y);
  for (c = string; *c != '\0'; c++)
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, *c);
  //glEnable(GL_LIGHTING);
}

// -- inicializacne funkcie

// nastavenie vsetko o texturach
// vracia true ak sa textura nacita spravne a nastavi, vracia false ak sa textura spravne nenacita
bool setupTextures()
{
	RGBIMG img;
	RGBIMG mur;
  RGBIMG podlaha;
  RGBIMG murNormalMap;
  RGBIMG murHeightMap;
  RGBIMG tehla;
  RGBIMG tehlaNormalMap;
  RGBIMG tehlaHeightMap;

	glGenTextures(POCET_TEXTUR, &textura_id[0]); // povie, ze chceme zostavit POCET_TEXTUR textur a ze ich chceme ulozit na prislusne indexy

	// nacitanie obrazkov
	if (!loadRgbImage("earth_flat_map.raw", 4096, 2048, &img))
    return false;
  if (!loadRgbImage("mur.raw", 512, 512, &mur))
    return false;
  if (!loadRgbImage("podlaha.raw", 512, 512, &podlaha))
    return false;
  if (!loadRgbImage("mur_NormalMap.raw", 256, 256, &murNormalMap))
    return false;
  if (!loadRgbImage("mur_Hauteur.raw", 256, 256, &murHeightMap))
    return false;
  if (!loadRgbImage("mur.raw"/*"brick.raw"*/, 512, 512, &tehla))
    return false;
  if (!loadRgbImage("mur_NormalMap.raw"/*"brickNormalMap.raw"*/, 256/*512*/, 256/*512*/, &tehlaNormalMap))
    return false;
  if (!loadRgbImage("mur_Hauteur.raw"/*"brickHeightMap.raw"*/, 256/*512*/, 256/*512*/, &tehlaHeightMap))
    return false;

  /** DEFINOVANIE TEXTURY 0  */
  glActiveTexture(GL_TEXTURE0); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_ZEME]);	// povie, ze textura na indexe 0 bude 2D textura
	// filtorvanie textury pri zvacsani a zmanseni
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvori texturu, 2D, hladina podrobnosti obrazku je obvikle 0, 3 farebne zlosky (RGB), sirka, vyska, ramcek nechavame nulovy, GL_RGB=obrazove data su tvorene red, green, blue v tomto poradi,,, GL_UNSIGNED_BYTE=data zloziek farieb su tvorene z bezznamienkovich byajtov, odkial vziat texturu
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, img.w, img.h, 0, GL_RGB, GL_UNSIGNED_BYTE, img.data);

  /** DEFINOVANIE TEXTURY 1  */
  glActiveTexture(GL_TEXTURE1); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_STIEN]);  // naviazem, povie, ze textura na indexe 1 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, mur.w, mur.h, 0, GL_RGB, GL_UNSIGNED_BYTE, mur.data);

  /** DEFINOVANIE TEXTURY 2  */
  glActiveTexture(GL_TEXTURE2); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_PODLAHY]);  // naviazem, povie, ze textura na indexe 2 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, podlaha.w, podlaha.h, 0, GL_RGB, GL_UNSIGNED_BYTE, podlaha.data);

  /** DEFINOVANIE TEXTURY 3  */
  glActiveTexture(GL_TEXTURE3); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_STIEN_NORMALOVA]);  // naviazem, povie, ze textura na indexe 3 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, murNormalMap.w, murNormalMap.h, 0, GL_RGB, GL_UNSIGNED_BYTE, murNormalMap.data);

  /** DEFINOVANIE TEXTURY 4  */
  glActiveTexture(GL_TEXTURE4); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_STIEN_VYSKOVA]);  // naviazem, povie, ze textura na indexe 4 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, murHeightMap.w, murHeightMap.h, 0, GL_RGB, GL_UNSIGNED_BYTE, murHeightMap.data);

  /** DEFINOVANIE TEXTURY 5  */
  glActiveTexture(GL_TEXTURE5); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_TEHLY]);  // naviazem, povie, ze textura na indexe 5 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, tehla.w, tehla.h, 0, GL_RGB, GL_UNSIGNED_BYTE, tehla.data);

  /** DEFINOVANIE TEXTURY 6  */
  glActiveTexture(GL_TEXTURE6); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_TEHLY_NORMALOVA]);  // naviazem, povie, ze textura na indexe 6 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, tehlaNormalMap.w, tehlaNormalMap.h, 0, GL_RGB, GL_UNSIGNED_BYTE, tehlaNormalMap.data);

  /** DEFINOVANIE TEXTURY 7 - DISPLACEMENT MAPPING*/
 /* glActiveTexture(GL_TEXTURE7); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_TEHLY_VYSKOVA]);  // naviazem, povie, ze textura na indexe 7 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA_FLOAT32_ATI, tehlaHeightMap.w, tehlaHeightMap.h, 0, GL_RGB, GL_UNSIGNED_BYTE, tehlaHeightMap.data);
*/
  /** DEFINOVANIE TEXTURY 7  */
  glActiveTexture(GL_TEXTURE7); // nastavim texturovaciu jednotku, ktorej bindujem texturu
  glBindTexture(GL_TEXTURE_2D, textura_id[TEXTURA_TEHLY_VYSKOVA]);  // naviazem, povie, ze textura na indexe 7 bude 2D textura
  // filtorvanie textury pri zvacsani a zmanseni
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	// vytvorenie textury
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, tehlaHeightMap.w, tehlaHeightMap.h, 0, GL_RGB, GL_UNSIGNED_BYTE, tehlaHeightMap.data);

	// obrazky su uz nacitane ako textury, data teda uvolnim
	delete img.data;
  delete mur.data;
  delete podlaha.data;
  delete murNormalMap.data;
  delete murHeightMap.data;
  delete tehla.data;
  delete tehlaNormalMap.data;
  delete tehlaHeightMap.data;

	return true;   // funkcia vracia true, ak spravne dobehne
}

void shaderInit()
{
  /****************************************************************
  1. vytvorit program shaderu
  2. pripojit prislusne programy shaderu k tomuto hlavnemu programu
  3. zlinkovat program shaderu
  4. overit, ci linkovanie dopadlo uspesne
  5. pouzit shader pre spracovanie vrcholu a frahmentu
  ****************************************************************/

  /** krok 2. vytvorim shader objekt (prazdny objekt, teda este nema pripojeny kod) */
  vertexSh = glCreateShader(GL_VERTEX_SHADER);
  fragmentSh = glCreateShader(GL_FRAGMENT_SHADER);
  vertexShOfTeapotTexture = glCreateShader(GL_VERTEX_SHADER);
  fragmentShOfTeapotTexture = glCreateShader(GL_FRAGMENT_SHADER);
  vertexShOfStien = glCreateShader(GL_VERTEX_SHADER);
  fragmentShOfStien = glCreateShader(GL_FRAGMENT_SHADER);
  vertexShOfPodlahy = glCreateShader(GL_VERTEX_SHADER);
  fragmentShOfPodlahy = glCreateShader(GL_FRAGMENT_SHADER);
  vertexShOfSimplePM = glCreateShader(GL_VERTEX_SHADER);
  fragmentShOfSimplePM = glCreateShader(GL_FRAGMENT_SHADER);
  vertexShOfSteepPM = glCreateShader(GL_VERTEX_SHADER);
  fragmentShOfSteepPM = glCreateShader(GL_FRAGMENT_SHADER);
  vertexShOfPOM = glCreateShader(GL_VERTEX_SHADER);
  fragmentShOfPOM = glCreateShader(GL_FRAGMENT_SHADER);
  vertexShOfPOMSS = glCreateShader(GL_VERTEX_SHADER);
  fragmentShOfPOMSS = glCreateShader(GL_FRAGMENT_SHADER);

  /** krok 3. linkovanie **/
  // krok 3.1 priradzuje shader objektu zdrojovy kod
  glShaderSource(vertexSh, 1, shaderSrcVertex, NULL);
  glShaderSource(fragmentSh, 1, shaderSrcFragment, NULL);
  glShaderSource(vertexShOfTeapotTexture, 1, shaderSrcVertexOfTeapotTexture, NULL);
  glShaderSource(fragmentShOfTeapotTexture, 1, shaderSrcFragmentOfTeapotTexture, NULL);
  glShaderSource(vertexShOfStien, 1, shaderSrcVertexOfStien, NULL);
  glShaderSource(fragmentShOfStien, 1, shaderSrcFragmentOfStien, NULL);
  glShaderSource(vertexShOfPodlahy, 1, shaderSrcVertexOfPodlahy, NULL);
  glShaderSource(fragmentShOfPodlahy, 1, shaderSrcFragmentOfPodlahy, NULL);
  glShaderSource(vertexShOfSimplePM, 1, shaderSrcVertexOfSimplePM, NULL);
  glShaderSource(fragmentShOfSimplePM, 1, shaderSrcFragmentOfSimplePM, NULL);
  glShaderSource(vertexShOfSteepPM, 1, shaderSrcVertexOfSteepPM, NULL);
  glShaderSource(fragmentShOfSteepPM,1, shaderSrcFragmentOfSeepPM, NULL);
  glShaderSource(vertexShOfPOM, 1, shaderSrcVertexOfPOM, NULL);
  glShaderSource(fragmentShOfPOM, 1, shaderSrcFragmentOfPOM, NULL);
  glShaderSource( vertexShOfPOMSS, 1, shaderSrcVertexOfPOMSS, NULL);
  glShaderSource( fragmentShOfPOMSS, 1, shaderSrcFragmentOfPOMSS, NULL);

  // krok 3.2 pripojeny zdrojovy kod zkompilujeme
  glCompileShader(vertexSh);
  glCompileShader(fragmentSh);
  glCompileShader(vertexShOfTeapotTexture);
  glCompileShader(fragmentShOfTeapotTexture);
  glCompileShader(vertexShOfStien);
  glCompileShader(fragmentShOfStien);
  glCompileShader(vertexShOfPodlahy);
  glCompileShader(fragmentShOfPodlahy);
  glCompileShader(vertexShOfSimplePM);
  glCompileShader(fragmentShOfSimplePM);
  glCompileShader(vertexShOfSteepPM);
  glCompileShader(fragmentShOfSteepPM);
  glCompileShader(vertexShOfPOM);
  glCompileShader(fragmentShOfPOM);
  glCompileShader( vertexShOfPOMSS);
  glCompileShader( fragmentShOfPOMSS);

  // krok 3.3 overit ci preklad nasho pripojeneho kodu pre vertex shader v objekte "vertexSh" prebehol spravne pomocou funkcie glGetShaderInfoLog
  //--------------kontorla prekladu, NEFUNGUJE

  // krok 3.4 objekty shaderov spojime do jedneho programu
  program = glCreateProgram(); // alokuje program
  programOfTeapotTexture = glCreateProgram(); // alokuje program
  programOfStien = glCreateProgram(); // alokuje program
  programOfPodlahy = glCreateProgram();
  programOfSimplePM = glCreateProgram();
  programOfSteepPM = glCreateProgram();
  programOfPOM = glCreateProgram();
  programOfPOMSS = glCreateProgram();

  glAttachShader(program, vertexSh); // pripojime objekty shadru do programu, ak by sme mali aj program pre fragment shader, tak by sme ho pripojili pomocou tej istej funkcie o riadok nizsie
  glAttachShader(program, fragmentSh);
  glAttachShader(programOfTeapotTexture, vertexShOfTeapotTexture);
  glAttachShader(programOfTeapotTexture, fragmentShOfTeapotTexture);
  glAttachShader(programOfStien, vertexShOfStien);
  glAttachShader(programOfStien, fragmentShOfStien);
  glAttachShader(programOfPodlahy, vertexShOfPodlahy);
  glAttachShader(programOfPodlahy, fragmentShOfPodlahy);
  glAttachShader(programOfSimplePM, vertexShOfSimplePM);
  glAttachShader(programOfSimplePM, fragmentShOfSimplePM);
  glAttachShader(programOfSteepPM, vertexShOfSteepPM);
  glAttachShader(programOfSteepPM, fragmentShOfSteepPM);
  glAttachShader(programOfPOM, vertexShOfPOM);
  glAttachShader(programOfPOM, fragmentShOfPOM);
  glAttachShader(programOfPOMSS, vertexShOfPOMSS);
  glAttachShader(programOfPOMSS, fragmentShOfPOMSS);

  // krok 3.5 konecne vsetky pripojene shader objekty zlinkujeme
  glLinkProgram(program);
  glLinkProgram(programOfTeapotTexture);
  glLinkProgram(programOfStien);
  glLinkProgram(programOfPodlahy);
  glLinkProgram(programOfSimplePM);
  glLinkProgram(programOfSteepPM);
  glLinkProgram(programOfPOM);
  glLinkProgram(programOfPOMSS);

  /** krok 4. overenie spravneho zlinkovania, ak zlinkovane spravne, ideme dalej **/
  // nepotrebne

  /** krok 5. a finalny program pre spracovanie vrcholov a frahmentov */
  // glUseProgram(program); // toto sa bude pouzivat uz iba pre konkretny objekt
  // a to tak, ze sa zavola glUseProgram() s prislusnym shader programom
  // na urcity objekt v display()
}

void init(void)
{
  glShadeModel(GL_SMOOTH); // abz sa farby miesali, GL_FLAT farby nemiesa, to je plochz model
  glClearColor(0.2, 0.1, 0.4, 0.5); // pozadie na modru
  glClearDepth(1.0f);// hlbkovy bafer
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);// typ hlbkoveho testovania

  // turn v-sync off
#if defined(__WIN32__) || defined(_WIN32)
  if (wglewIsSupported("WGL_EXT_swap_control"))
    wglSwapIntervalEXT(0);
#else
#endif

  if(setupTextures() == false) {nenacitana_texutra = true;} // nastavim texturovanie

  // nastavenie svetla 0
  glLightfv(GL_LIGHT0, GL_AMBIENT, Light0Ambient); // nastavenie okoliteho svetla 0
  glLightfv(GL_LIGHT0, GL_DIFFUSE, Light0Diffuse); // nastavenie priameho svetla 0
  glLightfv(GL_LIGHT0, GL_POSITION, Light0Position); // nastavenie pozicie svetla 0
  glLightfv(GL_LIGHT0, GL_SPECULAR, Light0Specular); // nastavenie spekularnej zlosky
  glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.2f); // osvetlenie bodu
  glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.00002f); //osvetlenie v dialke

  // nastavenie svetla 1
  glLightfv(GL_LIGHT1, GL_AMBIENT, Light1Ambient); // nastavenie okoliteho svetla 1
  glLightfv(GL_LIGHT1, GL_DIFFUSE, Light1Diffuse); // nastavenie priameho svetla 1
  glLightfv(GL_LIGHT1, GL_POSITION, Light1Position); // nastavenie pozicie svetla 1
  glLightfv(GL_LIGHT1, GL_SPECULAR, Light1Specular);
  glLightf(GL_LIGHT1, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 0.2f); // osvetlenie bodu
  glLightf(GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.00002f); //osvetlenie v dialke

  // nastavenie svetla 2
  glLightfv(GL_LIGHT2, GL_AMBIENT, Light2Ambient);
  glLightfv(GL_LIGHT2, GL_DIFFUSE, Light2Diffuse);
  glLightfv(GL_LIGHT2, GL_POSITION, Light2Position);
  glLightfv(GL_LIGHT2, GL_SPECULAR, Light2Specular);
  glLightf(GL_LIGHT2, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT2, GL_LINEAR_ATTENUATION, 0.2f);
  glLightf(GL_LIGHT2, GL_QUADRATIC_ATTENUATION, 0.00002f);

  // nastavenie svetla 3
  glLightfv(GL_LIGHT3, GL_AMBIENT, Light3Ambient);
  glLightfv(GL_LIGHT3, GL_DIFFUSE, Light3Diffuse);
  glLightfv(GL_LIGHT3, GL_POSITION, Light3Position);
  glLightfv(GL_LIGHT3, GL_SPECULAR, Light3Specular);
  glLightf(GL_LIGHT3, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT3, GL_LINEAR_ATTENUATION, 0.2f);
  glLightf(GL_LIGHT3, GL_QUADRATIC_ATTENUATION, 0.00002f);

  // nastavenie svetla 4
  glLightfv(GL_LIGHT4, GL_AMBIENT, Light4Ambient);
  glLightfv(GL_LIGHT4, GL_DIFFUSE, Light4Diffuse);
  glLightfv(GL_LIGHT4, GL_POSITION, Light4Position);
  glLightfv(GL_LIGHT4, GL_SPECULAR, Light4Specular);
  glLightf(GL_LIGHT4, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT4, GL_LINEAR_ATTENUATION, 0.2f);
  glLightf(GL_LIGHT4, GL_QUADRATIC_ATTENUATION, 0.00002f);

  // nastavenie svetla 5
  glLightfv(GL_LIGHT5, GL_AMBIENT, Light5Ambient);
  glLightfv(GL_LIGHT5, GL_DIFFUSE, Light5Diffuse);
  glLightfv(GL_LIGHT5, GL_POSITION, Light5Position);
  glLightfv(GL_LIGHT5, GL_SPECULAR, Light5Specular);
  glLightf(GL_LIGHT5, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT5, GL_LINEAR_ATTENUATION, 0.2f);
  glLightf(GL_LIGHT5, GL_QUADRATIC_ATTENUATION, 0.00002f);

  // nastavenie svetla 6
  glLightfv(GL_LIGHT6, GL_AMBIENT, Light6Ambient);
  glLightfv(GL_LIGHT6, GL_DIFFUSE, Light6Diffuse);
  glLightfv(GL_LIGHT6, GL_POSITION, Light6Position);
  glLightfv(GL_LIGHT6, GL_SPECULAR, Light6Specular);
  glLightf(GL_LIGHT6, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT6, GL_LINEAR_ATTENUATION, 0.2f);
  glLightf(GL_LIGHT6, GL_QUADRATIC_ATTENUATION, 0.00002f);

  // nastavenie svetla 7
  glLightfv(GL_LIGHT7, GL_AMBIENT, Light7Ambient);
  glLightfv(GL_LIGHT7, GL_DIFFUSE, Light7Diffuse);
  glLightfv(GL_LIGHT7, GL_POSITION, Light7Position);
  glLightfv(GL_LIGHT7, GL_SPECULAR, Light7Specular);
  glLightf(GL_LIGHT7, GL_CONSTANT_ATTENUATION, 0.6f);
  glLightf(GL_LIGHT7, GL_LINEAR_ATTENUATION, 0.2f);
  glLightf(GL_LIGHT7, GL_QUADRATIC_ATTENUATION, 0.00002f);

  // nastavenie materialovych vlastnosti
  glEnable(GL_COLOR_MATERIAL);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, MaterialAmbient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialDiffuse);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, MaterialSpecular);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mShininess);

  shaderInit();  // inicializacia shadrov
}

// -- zobrazovacie funkcie
void steny()
{
  int attrib_location1 = glGetAttribLocation(programOfStien, "tangent"); // attribute location

  glBegin(GL_QUADS);

    // Texture Coordinate (0,0) is top left.
    // lava
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 0.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 1.0);glVertex3f(-5.0, 0.0, -5.0);

    // lava super v predu
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 1.0);glVertex3f(-5.0, 0.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 0.0);glVertex3f(-5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);

    // zadna , TEDA toto texturovanie bol boj
    glNormal3f(0.0, 0.0, 1.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 1.0); glVertex3f(-5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 0.0); glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 0.0); glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 1.0); glVertex3f(5.0, 0.0, -5.0);

    // prava
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 1.0);glVertex3f(5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 0.0);glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 1.0);glVertex3f(5.0, 0.0, 5.0);

    // prava super v predu
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 1.0);glVertex3f(5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 0.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 0.0);glVertex3f(5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE1, 1.0, 1.0);glVertex3f(5.0, 0.0, 15.0);

  glEnd();
}

void podlaha()
{
  glBegin(GL_QUADS);

    // podlaha
    glNormal3f(0.0, 1.0, 0.0);
    glMultiTexCoord2f(GL_TEXTURE2, 0.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);
    glMultiTexCoord2f(GL_TEXTURE2, 0.0, 0.0);glVertex3f(-5.0, 0.0, -5.0);
    glMultiTexCoord2f(GL_TEXTURE2, 1.0, 0.0);glVertex3f(5.0, 0.0, -5.0);
    glMultiTexCoord2f(GL_TEXTURE2, 1.0, 1.0);glVertex3f(5.0, 0.0, 5.0);

    glNormal3f(0.0, 1.0, 0.0);
    glMultiTexCoord2f(GL_TEXTURE2, 0.0, 1.0);glVertex3f(-5.0, 0.0, 15.0);
    glMultiTexCoord2f(GL_TEXTURE2, 0.0, 0.0);glVertex3f(-5.0, 0.0, 5.0);
    glMultiTexCoord2f(GL_TEXTURE2, 1.0, 0.0);glVertex3f(5.0, 0.0, 5.0);
    glMultiTexCoord2f(GL_TEXTURE2, 1.0, 1.0);glVertex3f(5.0, 0.0, 15.0);

  glEnd();
}

void SimpleMappingTests(int attrib_location1)
{
  glBegin(GL_QUADS);

    // Texture Coordinate (0,0) is top left.
    // lava
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(-5.0, 0.0, -5.0);

    // lava 2
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(-5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(-5.0, 0.0, -15.0);

    // lava super v predu
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);

    // zadna
    glNormal3f(0.0, 0.0, 1.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, -15.0);

    // prava
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 5.0);

    // prava 2
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, -5.0);

    // prava super v predu
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 15.0);

    // spodna
    glNormal3f(0.0, 1.0, 0.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 5.0);

    // spodna 2
    glNormal3f(0.0, 1.0, 0.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, -5.0);

    // spodna super v predu
    glNormal3f(0.0, 1.0, 0.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 15.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 15.0);

    // horna
    glNormal3f(0.0, -1.0, 0.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 10.0, 5.0);

    // horna 2
    glNormal3f(0.0, -1.0, 0.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 10.0, -5.0);

    // horna super v predu
    glNormal3f(0.0, -1.0, 0.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 10.0, 15.0);

  glEnd();
}

void SteepMappingTests(int attrib_location1)
{
  glBegin(GL_QUADS);

    // Texture Coordinate (0,0) is top left.
    // lava
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(-5.0, 0.0, -5.0);

    // lava 2
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(-5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(-5.0, 0.0, -15.0);

    // lava super v predu
    glNormal3f(1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, -1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);

    // zadna
    glNormal3f(0.0, 0.0, 1.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, -15.0);

    // prava
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 5.0);

    // prava 2
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, -5.0);

    // prava super v predu
    glNormal3f(-1.0, 0.0, 0.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, 0, 0, 1);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 15.0);

    // spodna
    glNormal3f(0.0, 1.0, 0.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 5.0);

    // spodna 2
    glNormal3f(0.0, 1.0, 0.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, -5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 0.0, -15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, -5.0);

    // spodna super v predu
    glNormal3f(0.0, 1.0, 0.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 0.0, 15.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 0.0, 5.0);
    glVertexAttrib3f(attrib_location1, 1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 0.0, 15.0);

    // horna
    glNormal3f(0.0, -1.0, 0.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 10.0, 5.0);

    // horna 2
    glNormal3f(0.0, -1.0, 0.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 10.0, -5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, -15.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 10.0, -5.0);

    // horna super v predu
    glNormal3f(0.0, -1.0, 0.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 1.0);glVertex3f(-5.0, 10.0, 15.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 0.0, 0.0);glVertex3f(-5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 0.0);glVertex3f(5.0, 10.0, 5.0);
    glVertexAttrib3f(attrib_location1, -1, 0, 0);glMultiTexCoord2f(GL_TEXTURE5, 1.0, 1.0);glVertex3f(5.0, 10.0, 15.0);

  glEnd();
}

void display(void)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); //nieco cisti, ale nieviem co
  glLoadIdentity();
  glPushMatrix(); // ulozim aktualnu model wiew maticu
  gluLookAt(0.0, 2.0, 15.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0); //pohladova transformacia, prve3: kde lezi kamera, druhe3: kam sa kamera pozera, tretie3: vektor urcujuci kde je "hore"
  glTranslatef(0.0, -3.0, 0.0); //dalsia modelova transformacia (posun, rotacia, atd)
              // posuvam celu scenu o 3 dole
  if(scenka == 1)
  {
    // vykreslenie gule
    glUseProgram(programOfTeapotTexture);
    int texture_location = glGetUniformLocation(programOfTeapotTexture, "tex");
    glUniform1i(texture_location, 0);

      XYZ c;
      c.x = 0.0;
      c.y = 4.0;
      c.z = 0.0;
      CreateSphere( c, /*radius*/2.0, 300);

      glCallList(listGule);// vykreslim gulu, ktora je ulozena v display liste
    glUseProgram(0);

    // kompletny blok pre vykreslenie stien
    glUseProgram(programOfStien);
    int texture_location2 = glGetUniformLocation(programOfStien, "myTexture2");
    int texture_location4 = glGetUniformLocation(programOfStien, "normalMap");
    int texture_location5 = glGetUniformLocation(programOfStien, "heightMap");
    glUniform1i(texture_location2, 1);
    glUniform1i(texture_location4, 3);
    glUniform1i(texture_location5, 4);
      steny();
    glUseProgram(0);

    // kompletny blok pre vykreslenie podlahy
    glUseProgram(programOfPodlahy);
    int texture_location3 = glGetUniformLocation(programOfPodlahy, "myTexture3");
    glUniform1i(texture_location3, 2);
      podlaha();
    glUseProgram(0);

    // vykreslenie stihacky
    glUseProgram(program);
    glPushMatrix();
      glTranslatef(0.0, 1.0, 3.0);
      glScalef(5.0, 5.0, 5.0); // zvacsenie
      renderF14();
    glPopMatrix();
    glUseProgram(0);

    // vykreslenie helikoptery
    glUseProgram(program);
    glPushMatrix();
      glTranslatef(-3.0, 1.0, 2.0);
      glScalef(5.0, 5.0, 5.0); // zvacsenie
      glRotatef(180, 0.0, 1.0, 0.0);
      renderApacheAh64();
    glPopMatrix();
    glUseProgram(0);

    // vykreslenie raketoplanu
    glUseProgram(program);
    glPushMatrix();
      glTranslatef(3.0, 1.0, 3.5);
      glScalef(5.0, 5.0, 5.0); // zvacsenie
      glRotatef(-90.f, 1.f, 0.f, 0.f);
      renderShuttle();
    glPopMatrix();
    glUseProgram(0);

    glutSetWindowTitle("Scene 1: All test."); // nastavenie titulky okna
  }

  if(scenka == 2)
  {
    glUseProgram(programOfSimplePM);
    int attrib_location1 = glGetAttribLocation(programOfSimplePM, "tangent"); // attribute location
    int texture_location6 = glGetUniformLocation(programOfSimplePM, "stropColor");
    int texture_location7 = glGetUniformLocation(programOfSimplePM, "normalMap");
    int texture_location8 = glGetUniformLocation(programOfSimplePM, "heightMap");
    glUniform1i(texture_location6, 5);
    glUniform1i(texture_location7, 6);
    glUniform1i(texture_location8, 7);
      SimpleMappingTests(attrib_location1);
    glUseProgram(0);

    glutSetWindowTitle("Scene 2: Simple parallax mapping test."); // nastavenie titulky okna
  }

  if(scenka == 3)
  {
    glUseProgram(programOfSteepPM);
    int attrib_location1 = glGetAttribLocation(programOfSteepPM, "tangent"); // attribute location
    int texture_location6 = glGetUniformLocation(programOfSteepPM, "stropColor");
    int texture_location7 = glGetUniformLocation(programOfSteepPM, "normalMap");
    int texture_location8 = glGetUniformLocation(programOfSteepPM, "heightMap");
    glUniform1i(texture_location6, 5);
    glUniform1i(texture_location7, 6);
    glUniform1i(texture_location8, 7);
      SteepMappingTests(attrib_location1);
    glUseProgram(0);

    glutSetWindowTitle("Scene 3: Steep parallax mapping test."); // nastavenie titulky okna
  }

  if(scenka == 4)
  {
    glUseProgram(programOfPOM);
    int attrib_location1 = glGetAttribLocation(programOfPOM, "tangent"); // attribute location
    int texture_location6 = glGetUniformLocation(programOfPOM, "stropColor");
    int texture_location7 = glGetUniformLocation(programOfPOM, "normalMap");
    int texture_location8 = glGetUniformLocation(programOfPOM, "heightMap");
    glUniform1i(texture_location6, 5);
    glUniform1i(texture_location7, 6);
    glUniform1i(texture_location8, 7);
      SteepMappingTests(attrib_location1);
    glUseProgram(0);

    glutSetWindowTitle("Scene 4: Parallax occlusion mapping test."); // nastavenie titulky okna
  }

  if(scenka == 5)
  {
    glUseProgram(programOfPOMSS);
    int attrib_location1 = glGetAttribLocation(programOfPOMSS, "tangent"); // attribute location
    int texture_location6 = glGetUniformLocation(programOfPOMSS, "stropColor");
    int texture_location7 = glGetUniformLocation(programOfPOMSS, "normalMap");
    int texture_location8 = glGetUniformLocation(programOfPOMSS, "heightMap");
    glUniform1i(texture_location6, 5);
    glUniform1i(texture_location7, 6);
    glUniform1i(texture_location8, 7);
      SteepMappingTests(attrib_location1);
    glUseProgram(0);

    glutSetWindowTitle("Scene 5: Parallax occlusion mapping with selfshadowing test."); // nastavenie titulky okna
  }

  /** rotacia svetla 0 *********/
  cosx0 = cos(x0) * 4; // vypocet cosinu x-ovej os
  x0 += 0.1; // printf("x0 = %f \n", cosx0); // vypisovanie hodnoty cosinu
  cosz0 = cos(z0) * 4; // vypocet cosinu z-etovej osi
  z0 += 0.1; // printf("z0 = %f \n", cosz0); // vypisovanie hodnoty cosinu

  // nahratie novych pozicii do pola Light0Position a opakovane urcovanie pozicie svetla 0
  Light0Position[0] = cosx0;
  Light0Position[2] = cosz0;
  glLoadIdentity();
  glLightfv(GL_LIGHT0, GL_POSITION, Light0Position);

  /** rotacia svetla 1 *********/
  cosx1 = cos(x1) * 4; // vypocet cosinu x-ovej os
  x1 += 0.15;
  cosz1 = cos(z1) * 4;// vypocet cosinu z-etovej osi
  z1 += 0.15;

  // nahratie novych pozicii do pola Light1Position a opakovane urcovanie pozicie svetla 1
  Light1Position[0] = cosx1;
  Light1Position[2] = cosz1;
  glLoadIdentity();
  glLightfv(GL_LIGHT1, GL_POSITION, Light1Position);

  /** rotacia svetla 2 *********/
  cosy2 = cos(yy2) * 4;
  yy2 += 0.1;

  // nahratie novych pozicii do pola Light2Position a opakovane urcovanie pozicie svetla 2 s novymi poziciami
  Light2Position[1] = cosy2;
  glLoadIdentity();
  glLightfv(GL_LIGHT2, GL_POSITION, Light2Position);

  /** rotacia svetla 3 *********/
  cosx3 = -cos(x3) * 4; // vypocet cosinu x-ovej os
  x3 += 0.11;
  cosz3 = cos(z3) * 4; // vypocet cosinu z-etovej osi
  z3 += 0.11;

  // nahratie novych pozicii do pola Light3Position a opakovane urcovanie pozicie svetla 3 s novymi poziciami
  Light3Position[0] = cosx3;
  Light3Position[2] = cosz3;
  glLoadIdentity();
  glLightfv(GL_LIGHT3, GL_POSITION, Light3Position);

  /** rotacia svetla 4 *********/
  cosx4 = cos(x4) * 3; // vypocet cosinu x-ovej os
  x4 += 0.08;
  cosz4 = cos(z4) * 3; // vypocet cosinu z-etovej osi
  z4 += 0.08;

  // nahratie novych pozicii do pola Light4Position a opakovane urcovanie pozicie svetla 4 s novymi poziciami
  Light4Position[0] = cosx4;
  Light4Position[2] = cosz4;
  glLoadIdentity();
  glLightfv(GL_LIGHT4, GL_POSITION, Light4Position);

  /** rotacia svetla 5 *********/
  cosx5 = cos(x5) * 4; // vypocet cosinu x-ovej os
  x5 += 0.08;
  cosz5 = cos(z5) * 4; // vypocet cosinu z-etovej osi
  z5 += 0.1;

  // nahratie novych pozicii do pola Light5Position a opakovane urcovanie pozicie svetla 5 s novymi poziciami
  Light5Position[0] = cosx5;
  Light5Position[2] = cosz5;
  glLoadIdentity();
  glLightfv(GL_LIGHT5, GL_POSITION, Light5Position);

  /** rotacia svetla 6 *********/
  cosx6 = cos(x6) * 4; // vypocet cosinu x-ovej os
  x6 += 0.05;

  // nahratie novych pozicii do pola Light6Position a opakovane urcovanie pozicie svetla 6 s novymi poziciami
  Light6Position[0] = cosx6;
  glLoadIdentity();
  glLightfv(GL_LIGHT6, GL_POSITION, Light6Position);

  // vypocitanie casu medzi dvoma snimkami
  double pgettime = getTime();
  if (cas == -1)
  {
    cas = pgettime;
    rozdiel = 0.;
  }
  else
  {
    rozdiel = pgettime - cas;
    cas = pgettime;
  }

  // blok pre vypis FPS a helpu
  if( lastUpdate == -1.0 || pgettime - lastUpdate >= 0.25) // vypisujem kazdu stvrt sekundu
  {
    sprintf(FPS,"FPS: %f", 1/rozdiel);  // 1 / rozdiel == FPS
    lastUpdate = pgettime;
  }
  glMatrixMode(GL_PROJECTION); // prepnem do projekcnej matice
  glPushMatrix(); // pushnem projekcnu maticu
  glLoadIdentity(); // projekcnu maticu nastavim na identity (jednotkovu) maticu
  glOrtho(0,500/*win_w*/,500/*win_h*/,0,0,1); // ortogonalna projekcia
  glMatrixMode(GL_MODELVIEW); // prepem na modelovu a pohladovu maticu
  glLoadIdentity(); // modelovu a pohladovu maticu vyjednotkujem
  renderBitmapString(10.0, 15.0, 0.0, FPS); // zavolam pomocnu funkciu na vykreslovanie FPS
  if (toggle_help == true)
  {
    renderBitmapString(10.0, 79.0, 0.0, "HELP:");
    renderBitmapString(10.0, 91.0, 0.0, "Press h to toggle help.");
    renderBitmapString(10.0, 103.0, 0.0, "Press i to toggle renderer information.");
    renderBitmapString(10.0, 115.0, 0.0, "Press 1 for scene 1: all test.");
    renderBitmapString(10.0, 127.0, 0.0, "Press 2 for scene 2: simple parallax mapping test.");
    renderBitmapString(10.0, 139.0, 0.0, "Press 3 for scene 3: steep parallax mapping test.");
    renderBitmapString(10.0, 151.0, 0.0, "Press 4 for scene 4: parallax occlusion mapping test.");
    renderBitmapString(10.0, 163.0, 0.0, "Press 5 for scene 5: parallax occlusion mapping with selfshadowing test.");
    renderBitmapString(10.0, 175.0, 0.0, "Press ESC to quit application.");
    renderBitmapString(10.0, 195.0, 0.0, "WARNING:");
    renderBitmapString(10.0, 207.0, 0.0, "Shown information and help do affect performance. For better results turn them off.");
  }
  if (nenacitana_texutra == true)
    renderBitmapString(120.0, 15.0, 0.0, "ONE OF TEXTURES WAS NOT CORRECTLY READ!");

  if (toggle_info == true)
  {
    renderBitmapString(10.0, 35.0, 0.0, "INFORMATION:");
    char renderer[1];
    char version[1];
    sprintf(renderer, "Renderer: %s", glGetString(GL_RENDERER));
    renderBitmapString(10.0, 47.0, 0.0, renderer);
    sprintf(version, "OpenGL version: %s", glGetString(GL_VERSION));
    renderBitmapString(10.0, 59.0, 0.0, version);
  }

  glMatrixMode(GL_PROJECTION); // prepnem do projekcnej matice
  glPopMatrix(); // popnem projekcnu maticu, ktoru som mal ulozenu

  // prepnutie naspat na vykreslovanie po vypisani FPS
  glMatrixMode(GL_MODELVIEW); // prepnem do model view matice
  glPopMatrix(); // a popnem si model view maticu, ktoru som si pushol na zaciatku funkcie dispaly()

  glutSwapBuffers();
}

void reshape(int w, int h)
{
  glViewport(0, 0, w, h); //zobrazovacia transformacia (popisuje okno)
  glMatrixMode(GL_PROJECTION); //prepnem do projekcnej matice, lebo reshape sa vykona ako prve
  glLoadIdentity();
  gluPerspective(45.0,(float)w/(float)h, 0.1, 100.0);
  glMatrixMode(GL_MODELVIEW); //po nastaveni projekcnej matice, prepem na modelovu a pohladovu maticu, lebo sa bude vykonavat procedura display
  glLoadIdentity();
}

// -- ovladacie funkcie (switching a main)
void switching(unsigned char key, int x, int y)
{
  switch(key)
  {
    case '1': scenka = 1; break;
    case '2': scenka = 2; break;
    case '3': scenka = 3; break;
    case '4': scenka = 4; break;
    case '5': scenka = 5; break;
    case 'h':
    case 'H': toggle_help = !toggle_help; break;
    case 'i':
    case 'I': toggle_info = !toggle_info; break;
    case 27: exit(0); break;
  }
}

int main(int argc, char *argv[])
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
  glutInitWindowSize(700, 700);
  glutInitWindowPosition(200, 100);
  glutCreateWindow("BP");

  glewInit(); //inicializacia kniznice glew, musi byt pred init(), v init() je inicializacia shaderov ktora s glew pracuje a teda treba aby bola kniznica glew aktivna
  init();

  glutReshapeFunc(reshape);

  glutKeyboardFunc(switching);

  glutDisplayFunc(display);
  glutIdleFunc(display);

  glutMainLoop();

  return 0;
}
